@@ -139,6 +139,16 @@ public:
         return write( fd, buf, count );
     }
 
+    static struct stat lstat( const char *file_name )
+    {
+	struct stat ret;
+
+	if( ::lstat( file_name, &ret )!=0 )
+	    throw rscerror("lstat failed", errno, file_name );
+
+	return ret;
+    }
+
     static struct stat stat( const char *file_name )
     {
         struct stat ret;
@@ -171,7 +181,7 @@ public:
     }
     static autofd dup( int filedes )
     {
-        return autofd( dup(filedes) );
+        return autofd( ::dup(filedes) );
     }
     static void rmdir( const char *pathname )
     {
