@@ -7,10 +7,15 @@
 #define DIRSEP_S "/"
 #else
 #define PACKAGE_NAME "rsyncrypto"
-#define PACKAGE_STRING "rsyncrypto 0.19"
+#define PACKAGE_STRING "rsyncrypto 1.06"
 #define DIRSEP_C '\\'
 #define DIRSEP_S "\\"
 #pragma warning ( disable : 4786 )
+
+typedef __int64 _off_t;
+typedef _off_t off_t;
+#define _OFF_T_DEFINED
+
 #endif
 
 #include <sys/types.h>
@@ -62,11 +67,6 @@
 
 #include <argtable2.h>
 
-#if !HAVE_LSTAT
-// Some platforms don't support "lstat" and links
-#define lstat stat
-#endif
-
 class rscerror {
     std::string msg;
     std::string sysmsg;
@@ -100,12 +100,17 @@ struct startup_options {
     struct arg_lit *help, *del, *delkey, *filelist, *fr, *fk, *noarch, *version;
     struct arg_lit *decrypt, *verbosity, *recurse, *changed;
     struct arg_int *keysize, *rollwin, *rollmin, *rollsens, *trim, *nenest;
+    struct arg_int *noatime, *mod_win;
     struct arg_file *gzip;
     struct arg_file *src, *dst, *key, *master, *nameenc;
     struct arg_rem *rem1;
     struct arg_end *end;
 
-    void *argtable[26];
+    void *argtable[27
+#if HAVE_NOATIME
+        +1
+#endif
+        ];
 
     startup_options()
     {
@@ -122,7 +127,8 @@ struct startup_options {
         argtable[i++]=decrypt=arg_lit0( "d", "decrypt", "Decrypt");
         argtable[i++]=recurse=arg_lit0( "r", "recurse",
                 "<src> <dst> and <keys> are directory names, and are processed recursively");
-        argtable[i++]=changed=arg_lit0( "c", "changed", "Only encrypt changed files. Requires -r");
+        argtable[i++]=changed=arg_lit0( "c", "changed", "Only encrypt changed files. Requires -r or --filelist");
+        argtable[i++]=mod_win=arg_int0( NULL, "modify-window", "<n>", "compare mod-times with reduced accuracy" );
         argtable[i++]=nameenc=arg_file0( "n", "name-encrypt", "translation_file", "Encrypt file names");
         argtable[i++]=nenest=arg_int0( NULL, "ne-nesting", "<n>", "set the hash directory tree depth when encrypting file names" );
         argtable[i++]=trim=arg_int0( NULL, "trim", "<n>",
@@ -131,6 +137,10 @@ struct startup_options {
         argtable[i++]=delkey=arg_lit0( NULL, "delete-keys", "Delete also the keys. Implies --delete");
         argtable[i++]=filelist=arg_lit0( NULL, "filelist",
                 "<src> is a list of file and directory names to process. \"-\" means read from stdin.");
+        noatime=arg_int0( NULL, "noatime", "<n>", "Level of O_NOATIME use" );
+#if HAVE_NOATIME
+        argtable[i++]=noatime;
+#endif
         argtable[i++]=keysize=arg_int0( "b", "keybits", "<n>", "Size of key to create. Encryption only");
         argtable[i++]=fr=arg_lit0( NULL, "fr",
                 "Force new rollover parameters, even if previous encryption used a different setting.");
