@@ -38,6 +38,22 @@ typedef HANDLE file_t;
 class autofd : public autohandle {
     mutable bool f_eof;
 
+    static FILETIME ut2ft( time_t unixtime )
+    {
+        ULARGE_INTEGER res;
+        static const ULONGLONG epoch_start=116444736000000000;
+
+        res.QuadPart=unixtime;
+        res.QuadPart*=10*1000*1000;
+        res.QuadPart+=epoch_start;
+
+        FILETIME ret;
+        
+        ret.dwHighDateTime=res.HighPart;
+        ret.dwLowDateTime=res.LowPart;
+
+        return ret;
+    }
     static time_t ft2ut( FILETIME ft )
     {
         // Converts FILETIME to time_t
@@ -156,12 +172,34 @@ public:
         return write( *static_cast<const autohandle *>(this), buf, count );
     }
 
+    static struct stat lstat( const char *file_name )
+    {
+        return autofd::stat( file_name );
+    }
     static struct stat stat( const char *file_name )
     {
         struct stat ret;
+        WIN32_FILE_ATTRIBUTE_DATA data;
+        if( !GetFileAttributesEx( file_name, GetFileExInfoStandard, &data ) )
+            throw rscerror("stat failed", Error2errno(GetLastError()), file_name);
 
-        if( ::stat( file_name, &ret )!=0 )
-            throw rscerror("stat failed", errno, file_name );
+        ZeroMemory( &ret, sizeof(ret) );
+        ret.st_atime=ft2ut(data.ftLastAccessTime);
+        ret.st_ctime=ft2ut(data.ftCreationTime);
+        ret.st_mtime=ft2ut(data.ftLastWriteTime);
+        ret.st_dev=0;
+        ret.st_rdev=0;
+        
+        if( data.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY ) {
+            ret.st_mode=S_IFDIR;
+        } else {
+            ret.st_mode=S_IFREG;
+        }
+
+        ret.st_nlink=1;
+        ret.st_size=data.nFileSizeHigh;
+        ret.st_size<<=32;
+        ret.st_size|=data.nFileSizeLow;
 
         return ret;
     }
@@ -177,11 +215,13 @@ public:
         ret.st_atime=ft2ut(info.ftLastAccessTime);
         ret.st_ctime=ft2ut(info.ftCreationTime);
         ret.st_dev=0; // For a device - handle. Otherwise 0
-        //ret.st_mode; // unix mode
+        ret.st_mode=S_IFREG; // unix mode
         ret.st_mtime=ft2ut(info.ftLastWriteTime);
         ret.st_nlink=static_cast<short>(info.nNumberOfLinks); // nlink
         ret.st_rdev=0; // same as dev
-        ret.st_size=info.nFileSizeLow;
+        ret.st_size=info.nFileSizeHigh;
+        ret.st_size<<=32;
+        ret.st_size|=info.nFileSizeLow;
 
         return ret;
     }
@@ -204,7 +244,16 @@ public:
             throw rscerror("Invalid whence given", EINVAL);
         }
 
-        return SetFilePointer( file, offset, NULL, dwMoveMethod );
+        LONG offsethigh, offsetlow;
+        offsetlow=static_cast<LONG>(offset);
+        offsethigh=static_cast<LONG>(offset>>32);
+        offsetlow=SetFilePointer( file, offsetlow, &offsethigh, dwMoveMethod );
+
+        offset=offsethigh;
+        offset<<=32;
+        offset|=offsetlow;
+
+        return offset;
     }
     off_t lseek( off_t offset, int whence ) const
     {
@@ -212,10 +261,20 @@ public:
     }
     static int utimes( const char *filename, const struct timeval tv[2])
     {
-        struct utimbuf buf;
-        buf.actime=tv[0].tv_sec;
-        buf.modtime=tv[1].tv_sec;
-        return utime(filename, &buf);
+        FILETIME modtime, accesstime;
+
+        accesstime=ut2ft(tv[0].tv_sec);
+        modtime=ut2ft(tv[1].tv_sec);
+
+        // The only function in Windows that sets file modification/access times does so for
+        // open files only, so we have no choice but to open the file for write access
+        autofd file(filename, O_WRONLY);
+        if( SetFileTime(file, NULL, &accesstime, &modtime ) )
+            return 0;
+        else {
+            errno=Error2errno(GetLastError());
+            return -1;
+        }
     }
     static autofd dup( int filedes )
     {
